# -*- coding: utf-8 -*-
from django.db.models import Count
from django.http import Http404
from django.shortcuts import render
from django.utils.translation import ugettext as _

from django.views.decorators.http import require_http_methods
from django.shortcuts import redirect
try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse

from .models import Competition, Season, Race, RankModel, Driver, Team, get_tuples_from_results
from .records import get_record_config, get_record_label_dict
from .punctuation import get_punctuation_label_dict
from django.shortcuts import render, get_object_or_404

from .common import DRIVER27_NAMESPACE
from .decorators import competition_request

def competition_view(request, competition_slug):
    competition_obj = get_object_or_404(Competition, slug=competition_slug)
    title = '{competition}'.format(competition=competition_obj)
    context = {'competition': competition_obj, 'title': title}
    tpl = 'driver27/competition/view.html'
    return render(request, tpl, context)


def global_view(request):
    title = _('Global view')
    context = {'title': title}
    tpl = 'driver27/global/view.html'
    return render(request, tpl, context)

def global_tpl(request, competition=None):
    seasons = Season.objects
    if competition:
        seasons = seasons.filter(competition=competition)
    seasons = seasons.order_by('year', 'competition')
    context = {'seasons': seasons}
    tpl = 'driver27/global/_season_list.html'
    return render(request, tpl, context)

@competition_request
def season_view(request, context):
    season = context.get('season')
    title = '{competition}/{year:d}'.format(competition=season.competition, year=season.year)
    context.update(title=title)
    tpl = 'driver27/season/view.html'
    return render(request, tpl, context)


@competition_request
def _rank_view(request, context, rank_model='driver', by_season=False):
    season_or_competition = context.get('season_or_competition')
    by_season = request.POST.get('by_season', by_season)
    scoring_code = request.POST.get('scoring', None)

    punctuation_selector = get_punctuation_label_dict()
    rank_title = {'driver': _('DRIVERS'), 'team': _('TEAMS')}.get(rank_model)
    if rank_title is None:
        raise Http404(_('Impossible rank'))

    tpl = 'driver27/' + rank_model + '/list.html'
    title = u'{season_or_competition} [{title}]'.format(season_or_competition=season_or_competition,
                                                        title=rank_title)

    context.update(title=title, scoring_list=punctuation_selector, scoring_code=scoring_code, by_season=by_season)
    return render(request, tpl, context)

@competition_request
def driver_comeback_view(request, context, *args, **kwargs):
    season_or_competition = context['season_or_competition']
    rank_title = _('DRIVERS Comeback')

    title = u'{season_or_competition} [{title}]'.format(season_or_competition=season_or_competition,
                                                        title=rank_title)
    context['title'] = title
    tpl = 'driver27/driver/comeback.html'
    return render(request, tpl, context)


@competition_request
def common_olympic_view(request, context, tpl, olympic_method, rank_title):
    season_or_competition = context.get('season_or_competition')

    title = u'{season_or_competition} [{title}]'.format(season_or_competition=season_or_competition,
                                                        title=rank_title)

    context.update(title=title, olympic=True)
    return render(request, tpl, context)

def driver_olympic_view(request, *args, **kwargs):
    return common_olympic_view(request,
                               tpl='driver27/driver/list.html',
                               olympic_method='olympic_rank',
                               rank_title=_('DRIVERS rank by olympic mode'),
                               *args, **kwargs)


def team_olympic_view(request, *args, **kwargs):
    return common_olympic_view(request,
                               tpl='driver27/team/list.html',
                               olympic_method='team_olympic_rank',
                               rank_title=_('TEAMS rank by olympic mode'),
                               *args, **kwargs)

def driver_season_pos_view(request, competition_slug, year):
    season = get_object_or_404(Season, competition__slug=competition_slug, year=year)
    # rank = season.get_positions_draw()
    rank_title = 'POSITION draw'
    title = u'{season} [{title}]'.format(season=season,
                                         title=rank_title)
    context = {
        'season': season,
        'title': title,
        'positions': list(season.past_races.values_list('round', flat=True)),
        'olympic': True}
    return render(request, 'driver27/driver/list.html', context)


def race_list(request, competition_slug, year):
    season = get_object_or_404(Season, competition__slug=competition_slug, year=year)
    races = season.races.all()
    title = _('%(season)s [RACES]') % {'season': season}
    context = {'races': races, 'season': season, 'title': title}
    tpl = 'driver27/race/list.html'
    return render(request, tpl, context)


def race_view(request, competition_slug, year, race_id=None):
    race = get_object_or_404(Race, season__competition__slug=competition_slug, season__year=year, round=race_id)
    results = race.results.all() \
        .annotate(null_position=Count('finish')).order_by('-null_position', 'finish', 'qualifying')
    title = _('Results of %(race)s') % {'race': race}
    context = {'race': race, 'season': race.season, 'title': title, 'results': results}
    tpl = 'driver27/race/view.html'
    return render(request, tpl, context)


def get_safe_record_config(record):
    record_config = get_record_config(record)
    if record_config:
        return record_config
    else:
        raise Http404(_('Record does not exist'))

@competition_request
def get_record_common_context(request, context, record=None, *args, **kwargs):
    season_or_competition = context.get('season_or_competition')

    context.update(record=record)
    if record:
        record_config = get_safe_record_config(record)
        record_label = record_config.get('label')
        title = _('%(record_label)s Record, %(season_or_competition)s') \
                % {'record_label': record_label, 'season_or_competition': season_or_competition}
        context['record_filter'] = record_config.get('filter')
    else:
        title = _('Select a %(season_or_competition)s record') % {'season_or_competition': season_or_competition}

    context['title'] = title
    context.pop('record_filter', None)
    context['record_codes'] = get_record_label_dict()
    return context

def driver_record_view(request, *args, **kwargs):
    context = get_record_common_context(request, *args, **kwargs)
    tpl = 'driver27/driver/record.html'
    return render(request, tpl, context)



def team_record_view(request, rank_type='STATS', *args, **kwargs):
    context = get_record_common_context(request, *args, **kwargs)
    context['rank_opt'] = rank_type
    context['doubles_record_codes'] = [double_code for double_code, double_label in get_record_label_dict(doubles=True)]
    tpl = 'driver27/team/record.html'
    return render(request, tpl, context)


def driver_profile_view(request, driver_id):
    driver = get_object_or_404(Driver, pk=driver_id)
    by_season = driver.get_stats_by_season(append_points=True)
    by_competition = driver.get_stats_by_competition(append_points=True)
    context = {
        'driver': driver,
        'by_season': by_season,
        'by_competition': by_competition,
        'results': get_tuples_from_results(driver.get_results()),
        'stats': driver.get_stats_list(append_points=True),
        'title': 'Profile of {driver}'.format(driver=driver)
    }
    tpl = 'driver27/driver/profile.html'
    return render(request, tpl, context)


def team_profile_view(request, team_id):
    team = get_object_or_404(Team, pk=team_id)
    by_season = team.get_stats_by_season(append_points=True)
    by_competition = team.get_stats_by_competition(append_points=True)
    context = {
        'team': team,
        'by_season': by_season,
        'by_competition': by_competition,
        'stats': team.get_stats_list(append_points=True),
        'title': 'Profile of {team}'.format(team=team)
    }
    tpl = 'driver27/team/profile.html'
    return render(request, tpl, context)




from setuptools import setup, find_packages

with open('requirements.txt') as f:
     requirements = f.read().splitlines()

readme = open('README.rst', 'r')
README_TEXT = readme.read()
readme.close()

setup(
    name='django-driver27',
    version='0.24.2.1',
    include_package_data=True,
    packages=find_packages(),
    url='https://github.com/SRJ9/django-driver27.git',
    download_url='https://github.com/SRJ9/django-driver27/archive/develop.zip',
    license='MIT',
    author='Jose ER',
    author_email='srj9es@gmail.com',
    description='Racing competition manager based on Django',
    install_requires=requirements,
    long_description=README_TEXT,
    keywords=['Django', 'motorsport', 'formula one', 'formula 1', 'f1', 'manager'],
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stablegit
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        #
        # # Specify the Python versions you support here. In particular, ensure
        # # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
    ]

)



# -*- coding: utf-8 -*-
from django import template
from ..models import Race
register = template.Library()

@register.filter
def race_url(race_id):
    race = Race.objects.get(pk=race_id)
    return race.get_absolute_url()



# -*- coding: utf-8 -*-
from django.test import TestCase
from .common import CommonSeatTestCase
from ..models import Seat


class SeatTestCase(TestCase, CommonSeatTestCase):
    def test_seat_unicode(self):
        seat = self.get_test_seat()
        driver = seat.driver
        team = seat.team
        expected_seat = '{driver} in {team}'.format(driver=driver, team=team)
        self.assertEquals(str(seat), expected_seat)






from django.core.management.base import BaseCommand, CommandError
from driver27.models import Driver, Team, Seat
import csv

class ImportOptionException(Exception):
    pass

class Command(BaseCommand):
    help = 'Import driver/team/seat from CSV'

    @staticmethod
    def create_entry(row, import_opt):
        invalid_keywords = []

        import_opts = {
            'drivers': Driver,
            'teams': Team,
            'seats': Seat
        }

        model_cls = import_opts.get(import_opt)
        if model_cls is None:
            raise ImportOptionException('Import param is invalid')

        model_fields = [f.column for f in model_cls._meta.get_fields() if hasattr(f, 'column')]

        for x in row:
            if x not in model_fields:
                invalid_keywords.append(x)

        for invalid_keyword in invalid_keywords:
            del row[invalid_keyword]
        if not row:
            return False
        return model_cls.objects.create(**row)

    def add_arguments(self, parser):
        parser.add_argument('csv',)
        parser.add_argument('import')

    def handle(self, *args, **options):
        with open(options['csv'], 'rb') as csvfile:
            reader = csv.DictReader(csvfile, delimiter=',')
            for row in reader:
                self.create_entry(row, options['import'])



from django.conf import settings
from django.db.models import F
from django.utils.translation import ugettext as _


def get_init_config():
    init_config = {
        'RECORDS': {
            'RACE': {'label': _('Race'), 'filter': {}},
            'POLE': {'label': _('Pole'), 'filter': {'qualifying__exact': 1}},
            'POLE-OUT': {'label': _('Pole and Out'), 'filter': {'qualifying__exact': 1,
                                                                'retired': True}},
            'FIRST-ROW': {'label': _('First row'), 'filter': {'qualifying__gte': 1, 'qualifying__lte': 2},
                          'doubles': True},
            'COMEBACK-TO-TEN': {'label': _('Comeback to 10 firsts'), 'filter': {'qualifying__gte': 11,
                                                                                'finish__gte': 1,
                                                                                'finish__lte': 10},
                                'doubles': True},
            'COMEBACK-TO-WIN': {'label': _('Comeback to win'), 'filter': {'qualifying__gt': 1,
                                                                          'finish': 1}},
            'COMEBACK': {'label': _('Comeback'), 'filter': {'finish__lt': F('qualifying'), 'retired': False,
                                                            'finish__gt': 0}},
            'PODIUM-AFTER-11': {'label': _('Podium after qualifying 11th or more'),
                                'filter': {'qualifying__gte': 11, 'finish__gte': 1, 'finish__lte': 3},
                                'doubles': True},
            'WIN': {'label': _('Win'), 'filter': {'finish__exact': 1}},
            'POLE-WIN': {'code': 'POLE-WIN', 'label': _('Pole and Win'), 'filter': {'qualifying__exact': 1,
                                                                                    'finish__exact': 1}},
            'POLE-WIN-FL': {'label': _('Pole, Win, Fastest lap'), 'filter': {'qualifying__exact': 1,
                                                                             'finish__exact': 1,
                                                                             'race__fastest_car': F('seat')}},
            'FASTEST': {'label': _('Fastest lap'), 'filter': {'race__fastest_car': F('seat')}},
            'FIRST-TWO': {'label': _('Finish 1st or 2nd'), 'filter': {'finish__gte': 1, 'finish__lte': 2},
                          'doubles': True},
            'PODIUM': {'label': _('Podium'), 'filter': {'finish__gte': 1, 'finish__lte': 3}, 'doubles': True},
            'OUT': {'label': _('OUT'), 'filter': {'retired': True}, 'doubles': True},
            'CHECKERED-FLAG': {'label': _('Checkered Flag'), 'filter': {'retired': False}, 'doubles': True},
            'TOP5': {'label': _('Top 5'), 'filter': {'finish__gte': 1, 'finish__lte': 5}, 'doubles': True},
            'TOP10': {'code': 'TOP10', 'label': _('Top 10'), 'filter': {'finish__gte': 1, 'finish__lte': 10},
                      'doubles': True},
        },
        'PUNCTUATION': {
            'F1-25': {'type': 'full', 'finish': [25, 18, 15, 12, 10, 8, 6, 4, 2, 1], 'fastest_lap': 0,
                      'label': 'F1 (25p 1st)'},
            'F1-10+8': {'type': 'full', 'finish': [10, 8, 6, 4, 3, 2, 1], 'fastest_lap': 0,
                        'label': 'F1 (10p 1st, 8p 2nd)'},
            'F1-10+6': {'type': 'full', 'finish': [10, 6, 4, 3, 2, 1], 'fastest_lap': 0,
                        'label': 'F1 (10p 1st, 6p 2nd)'},
            'F1-62-90': {'type': 'full', 'finish': [9,6,4,3,2,1], 'fastest_lap': 0, 'label': 'F1 (9p 1st - from 62 to 90)'},
            'MotoGP': {'type': 'full', 'finish': [25, 20, 16, 13, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1], 'fastest_lap': 0,
                       'label': 'Moto GP'},
            'MotoGP-92': {'type': 'full', 'finish': [20, 15, 12, 10, 8, 6, 4, 3, 2, 1], 'fastest_lap': 0,
                          'label': 'Moto GP (only 1992)'},
            'MotoGP-88-91': {'code': 'MotoGP-88-91', 'type': 'full',
                             'finish': [20, 17, 15, 13, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1], 'fastest_lap': 0,
                             'label': 'Moto GP (1988-91)'},
            'MotoGP-77-87': {'type': 'full', 'finish': [15, 12, 10, 8, 6, 6, 5, 3, 2, 1], 'fastest_lap': 0,
                             'label': 'Moto GP (1977-87)'},
        }
    }

    return init_config


def get_settings_config():
    config_key = 'DR27_CONFIG'
    config = {}
    if hasattr(settings, config_key):
        config.update(getattr(settings, config_key))
    return config


def get_config(param, key=None):
    config = {}
    init_config = get_init_config().get(param)
    if init_config:
        config.update(init_config)
    settings_config = get_settings_config().get(param)
    if settings_config:
        config.update(settings_config)
    if key:
        return config.get(key)
    return config



from django.contrib import admin
from django.db.models import Q
from ..models import Team, GrandPrix, Seat, Race


class CompetitionFilterInline(admin.TabularInline):
    def get_formset(self, request, obj=None, **kwargs):
        formset = super(CompetitionFilterInline, self).get_formset(request, obj, **kwargs)
        formset.request = request
        return formset

    @staticmethod
    def seat_for_foreignkey(obj):
        if isinstance(obj, Race):
            return obj.seats.all()
        else:
            seat_filter = {'contender__competition__exact': obj.competition}
            raise Exception('NO controller')

    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        if getattr(request, '_obj_', None):
            obj = request._obj_
            if db_field.name == 'team' and not isinstance(obj, Team):
                kwargs['queryset'] = Team.objects.filter(competitions__exact=obj.competition)
            elif db_field.name == 'grand_prix':
                kwargs['queryset'] = GrandPrix.objects.filter(competitions__exact=obj.competition)
            elif db_field.name == 'seat' and not isinstance(obj, Seat):
                kwargs['queryset'] = self.seat_for_foreignkey(obj)
        return super(CompetitionFilterInline, self).formfield_for_foreignkey(db_field, request, **kwargs)
